"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.validateEvents = exports.getEventStore = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _eventstoreLite = _interopRequireDefault(require("@resolve-js/eventstore-lite"));

var _eventsUtils = require("./events-utils");

var _constants = require("../constants");

var _excluded = ["events"];

var validateEvents = function validateEvents(events) {
  var stampedCount = events.filter(function (event) {
    return event.timestamp != null;
  }).length;

  if (stampedCount !== events.length && stampedCount > 0) {
    throw Error(_constants.ambiguousEventsTimeErrorMessage);
  }

  var invalidEventIndex = events.findIndex(function (event) {
    var _event$payload;

    return ((_event$payload = event.payload) == null ? void 0 : _event$payload[_constants.reservedEventOrderField]) != null;
  });

  if (invalidEventIndex >= 0) {
    throw Error((0, _constants.getReservedFieldUsedErrorMessage)(_constants.reservedEventOrderField));
  }
};

exports.validateEvents = validateEvents;

var getEventStore = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(events, context) {
    var preparedEvents, eventByAggregateIdVersion, eventIndex, _preparedEvents$event, aggregateId, aggregateVersion, event, eventStoreAdapter;

    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            validateEvents(events);
            preparedEvents = (0, _eventsUtils.prepareEvents)(events, context);
            eventByAggregateIdVersion = new Map();

            for (eventIndex = 0; eventIndex < events.length; eventIndex++) {
              _preparedEvents$event = preparedEvents[eventIndex], aggregateId = _preparedEvents$event.aggregateId, aggregateVersion = _preparedEvents$event.aggregateVersion;
              event = events[eventIndex];

              if (event != null) {
                eventByAggregateIdVersion.set(aggregateId + ":" + aggregateVersion, event);
              }
            }

            eventStoreAdapter = (0, _eventstoreLite["default"])({
              databaseFile: ':memory:'
            });
            _context2.next = 7;
            return eventStoreAdapter.init();

          case 7:
            _context2.next = 9;
            return Promise.all(preparedEvents.map(function (event) {
              return eventStoreAdapter.saveEvent(event);
            }));

          case 9:
            return _context2.abrupt("return", (0, _extends2["default"])({}, eventStoreAdapter, {
              loadEvents: function () {
                var _loadEvents = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
                  var _yield$eventStoreAdap,
                      storedEvents,
                      other,
                      events,
                      timestampIndex,
                      _eventByAggregateIdVe,
                      _eventByAggregateIdVe2,
                      _event,
                      _args = arguments;

                  return _regenerator["default"].wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          _context.next = 2;
                          return eventStoreAdapter.loadEvents.apply(eventStoreAdapter, _args);

                        case 2:
                          _yield$eventStoreAdap = _context.sent;
                          storedEvents = _yield$eventStoreAdap.events;
                          other = (0, _objectWithoutPropertiesLoose2["default"])(_yield$eventStoreAdap, _excluded);
                          events = (0, _eventsUtils.stripEvents)(JSON.parse(JSON.stringify(storedEvents)));

                          for (timestampIndex = 0; timestampIndex < events.length; timestampIndex++) {
                            _event = events[timestampIndex];
                            _event.timestamp = (_eventByAggregateIdVe = (_eventByAggregateIdVe2 = eventByAggregateIdVersion.get(_event.aggregateId + ":" + _event.aggregateVersion)) == null ? void 0 : _eventByAggregateIdVe2.timestamp) !== null && _eventByAggregateIdVe !== void 0 ? _eventByAggregateIdVe : timestampIndex;
                          }

                          return _context.abrupt("return", (0, _extends2["default"])({}, other, {
                            events: events
                          }));

                        case 8:
                        case "end":
                          return _context.stop();
                      }
                    }
                  }, _callee);
                }));

                function loadEvents() {
                  return _loadEvents.apply(this, arguments);
                }

                return loadEvents;
              }()
            }));

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getEventStore(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.getEventStore = getEventStore;
//# sourceMappingURL=get-event-store.js.map